﻿#region License
/*
FlagsISO

A library containing country (and some other) flags.
Copyright (C) 2015 VPKSoft, Petteri Kautonen

Contact: vpksoft@vpksoft.net

This file is part of FlagsISO.

FlagsISO is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

FlagsISO is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with FlagsISO.  If not, see <http://www.gnu.org/licenses/>.
*/
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;
using System.Globalization;
using System.Reflection;
using System.Resources;

namespace FlagsISO
{
    /// <summary>
    /// A size of the flag in pixels (NxN).
    /// </summary>
    public enum FlagSizes
    {
        /// <summary>
        /// 16x16 bitmap.
        /// </summary>
        Size_x_16,
        /// <summary>
        /// 24 x 24 bitmap.
        /// </summary>
        Size_x_24,
        /// <summary>
        /// 32x32 bitmap.
        /// </summary>
        Size_x_32,
        /// <summary>
        /// 48x48 bitmap.
        /// </summary>
        Size_x_48,
        /// <summary>
        /// 64x64 bitmap.
        /// </summary>
        Size_x_64
    };

    /// <summary>
    /// A class to get coutry flags to your application.
    /// </summary>
    public class CountryFlagsISO
    {
        /// <summary>
        /// Gets a bitmap of a country flag.
        /// </summary>
        /// <param name="culture">CultureInfo class instance to use to get the flag.</param>
        /// <param name="flagSize">A FlagSizes enumeration which tells the size of the flag.</param>
        /// <param name="shiny">If true a a flag is returned with "lovely glossy finish".</param>
        /// <returns>A Bitmap containing the requested flag or null if the flag was not found.</returns>
        public static Bitmap GetForCountry(CultureInfo culture, FlagSizes flagSize, bool shiny)
        {
            RegionInfo ri = new RegionInfo(culture.Name);
            return GetForCountry(ri, flagSize, shiny);
        }

        /// <summary>
        /// Gets a bitmap of a country flag.
        /// </summary>
        /// <param name="culture">CultureInfo class instance to use to get the flag.</param>
        /// <param name="flagSize">A FlagSizes enumeration which tells the size of the flag.</param>
        /// <returns>A Bitmap containing the requested flag or null if the flag was not found.</returns>
        public static Bitmap GetForCountry(CultureInfo culture, FlagSizes flagSize)
        {
            RegionInfo ri = new RegionInfo(culture.Name);
            return GetForCountry(ri, flagSize, false);
        }

        /// <summary>
        /// Gets a bitmap of a country flag.
        /// </summary>
        /// <param name="ri">A RegionInfo class instance to use to get the flag.</param>
        /// <param name="flagSize">A FlagSizes enumeration which tells the size of the flag.</param>
        /// <param name="shiny">If true a a flag is returned with "lovely glossy finish".</param>
        /// <returns>A Bitmap containing the requested flag or null if the flag was not found.</returns>
        public static Bitmap GetForCountry(RegionInfo ri, FlagSizes flagSize, bool shiny)
        {
            return GetForCountry(ri.TwoLetterISORegionName, flagSize, shiny);
        }

        /// <summary>
        /// Gets a bitmap of a country flag.
        /// </summary>
        /// <param name="ri">A RegionInfo class instance to use to get the flag.</param>
        /// <param name="flagSize">A FlagSizes enumeration which tells the size of the flag.</param>
        /// <returns>A Bitmap containing the requested flag or null if the flag was not found.</returns>
        public static Bitmap GetForCountry(RegionInfo ri, FlagSizes flagSize)
        {
            return GetForCountry(ri.TwoLetterISORegionName, flagSize, false);
        }

        /// <summary>
        /// Gets a bitmap of a country flag.
        /// </summary>
        /// <param name="name">A two letter ISO country name.</param>
        /// <param name="flagSize">A FlagSizes enumeration which tells the size of the flag.</param>
        /// <param name="shiny">If true a a flag is returned with "lovely glossy finish".</param>
        /// <returns>A Bitmap containing the requested flag or null if the flag was not found.</returns>
        public static Bitmap GetForCountry(string name, FlagSizes flagSize, bool shiny)
        {
            string resBitmapName = shiny ? "shiny" : "flat";
            if (flagSize == FlagSizes.Size_x_16)
            {
                resBitmapName += "_16x";
            }
            else if (flagSize == FlagSizes.Size_x_24)
            {
                resBitmapName += "_24x";
            }
            else if (flagSize == FlagSizes.Size_x_32)
            {
                resBitmapName += "_32x";
            }
            else if (flagSize == FlagSizes.Size_x_48)
            {
                resBitmapName += "_48x";
            }
            else if (flagSize == FlagSizes.Size_x_64)
            {
                resBitmapName += "_64x";
            }
            resBitmapName += name;
            return (Bitmap)FlagIcons.ResourceManager.GetObject(resBitmapName);
        }

        /// <summary>
        /// Gets a bitmap of a country flag.
        /// </summary>
        /// <param name="name">A two letter ISO country name.</param>
        /// <param name="flagSize">A FlagSizes enumeration which tells the size of the flag.</param>
        /// <returns>A Bitmap containing the requested flag or null if the flag was not found.</returns>
        public static Bitmap GetForCountry(string name, FlagSizes flagSize)
        {
            return GetForCountry(name, flagSize, false);
        }

        /// <summary>
        /// A list of country codes in upper case which country
        /// <para/>flags are included in this library. The codes
        /// <para/>are in two letter ISO country names except for
        /// <para/>few that aren't considered as countries or don't
        /// <para/>have globally accepted country status or is't
        /// <para/>a country (e.g. "_united_nations").
        /// </summary>
        /// <returns>A list of strings in two letter ISO country names.</returns>
        public static List<string> GetCountries()
        {
            List<string> retVal = new List<string>();
            retVal.AddRange(FlagIcons.countryList.Split(Environment.NewLine.ToArray(), StringSplitOptions.RemoveEmptyEntries));
            retVal.Sort();
            return retVal;
        }

        /// <summary>
        /// Returns the country name by it's two letter ISO country name. 
        /// </summary>
        /// <param name="twoISO">Two letter ISO country name.</param>
        /// <returns>Country name</returns>
        public static string GetNameByTwoLetterISO(string twoISO)
        {
            string[] longCountryList = FlagIcons.countryListLong.Split(Environment.NewLine.ToCharArray(), StringSplitOptions.RemoveEmptyEntries);
            Dictionary<string, string> dict = new Dictionary<string, string>();
            for (int i = 0; i < longCountryList.Length; i++)
            {
                dict.Add(longCountryList[i].Split('=')[1], longCountryList[i].Split('=')[0]);
            }
            try
            {
                return dict[twoISO].Replace("-", " ");
            }
            catch
            {
                return string.Empty;
            }            
        }

        /// <summary>
        /// A list of regions of which are supported
        /// <para/>by the .NET Framework.
        /// </summary>
        /// <returns>A list of RegionInfo instances.</returns>
        public static List<RegionInfo> GetRegions()
        {
            List<RegionInfo> retVal = new List<RegionInfo>();
            CultureInfo[] ciArr = CultureInfo.GetCultures(CultureTypes.SpecificCultures);
            foreach (CultureInfo i in ciArr)
            {
                try
                {
                    retVal.Add(new RegionInfo(i.LCID));
                }
                catch
                {
                    // do nothin
                }
            }
            return retVal;
        }

        /// <summary>
        /// A list of cultures of which are supported
        /// <para/>by the .NET Framework that also can be converted
        /// <para/>to regions.
        /// </summary>
        /// <returns>A list of RegionInfo instances.</returns>        
        public static List<CultureInfo> GetCultures()
        {
            List<CultureInfo> retVal = new List<CultureInfo>();
            CultureInfo[] ciArr = CultureInfo.GetCultures(CultureTypes.SpecificCultures);
            foreach (CultureInfo i in ciArr)
            {
                try
                {
                    new RegionInfo(i.LCID);
                    retVal.Add(i);
                }
                catch
                {
                    // do nothin
                }
            }
            return retVal;
        }
    }
}
